@@ -10,10 +10,15 @@ from concurrent.futures import ThreadPoolExecutor
 from functools import singledispatch
 from typing import Any, List, Union
 
-from aiohttp import web
-from aiohttp.web import Response
+from aiohttp.web import HTTPException, Request, Response, json_response
+from pydantic import BaseModel
 
-from .odm import BaseModel, FaunaModel
+from aiofauna.json import FaunaJSONEncoder
+
+from .json import parse_json, to_json
+from .odm import FaunaModel
+
+T = typing.TypeVar("T")
 
 
 def asyncify(
@@ -34,7 +39,7 @@ def asyncify(
 
 def aio(max_workers: int = 5) -> typing.Callable[[typing.Type], typing.Type]:
     """Decorator that converts all the methods of a class into async methods."""
-    
+
     def decorator(cls: typing.Type) -> typing.Type:
         attrs: typing.Dict[str, typing.Any] = {}
         attrs["executor"] = ThreadPoolExecutor(max_workers=max_workers)
@@ -51,34 +56,6 @@ def aio(max_workers: int = 5) -> typing.Callable[[typing.Type], typing.Type]:
     return decorator
 
 
-def redirect(url):
-    """
-    Create an HTTP redirect response.
-
-    Args:
-        url (str): The URL to redirect to.
-
-    Returns:
-        web.HTTPFound: An HTTP redirect response with the specified URL.
-    """
-
-    return web.HTTPFound(url)
-
-
-def make_response(data, headers=None, status=200):
-    """
-    Create an HTTP response with the given data, headers, and status code.
-
-    Args:
-        data (str): The response body data.
-        headers (dict, optional): A dictionary of headers to include in the response. Defaults to None.
-        status (int, optional): The HTTP status code for the response. Defaults to 200.
-
-    Returns:
-        web.Response: An HTTP response with the specified data, headers, and status code.
-    """
-    return web.Response(body=data, headers=headers, status=status)
-
 @singledispatch
 def do_response(response: Any) -> Response:
     """
@@ -89,35 +66,26 @@ def do_response(response: Any) -> Response:
 
 @do_response.register(BaseModel)
 def _(response: BaseModel) -> Response:
-    return Response(
-        status=200, body=response.json(
-            exclude_none=True
-            ), content_type="application/json"
-    )
+    return json_response(response.dict(exclude_none=True), dumps=to_json)
 
 
 @do_response.register(FaunaModel)
 def _(response: FaunaModel) -> Response:
-    return Response(
-        status=200, text=response.json(
-            exclude_none=True
-            ), content_type="application/json"
-    )
+    return json_response(response.dict(), dumps=to_json)
 
 
 @do_response.register(dict)
 def _(response: dict) -> Response:
-    return Response(
-        status=200, body=json.dumps(response), content_type="application/json"
-    )
+    return json_response(response, dumps=to_json)
 
 
 @do_response.register(str)
 def _(response: str) -> Response:
-    if "<html>" in response:
+    if response.startswith("<") and response.endswith(">"):
         return Response(status=200, text=response, content_type="text/html")
     return Response(status=200, text=response, content_type="text/plain")
 
+
 @do_response.register(bytes)
 def _(response: bytes) -> Response:
     return Response(status=200, body=response, content_type="application/octet-stream")
@@ -140,19 +108,4 @@ def _(response: bool) -> Response:
 
 @do_response.register(list)
 def _(response: List[Union[FaunaModel, BaseModel, dict, str, int, float]]) -> Response:
-    processed_response = []
-
-    for item in response:
-        if isinstance(item, (FaunaModel, BaseModel)):
-            processed_response.append(item.json(exclude_none=True))
-        elif isinstance(item, dict):
-            processed_response.append(item)
-        elif isinstance(item, str):
-            processed_response.append(item)
-        elif isinstance(item, (int, float, bool)):
-            processed_response.append(str(item))
-        else:
-            raise TypeError(f"Cannot serialize type {type(item)}")
-    return Response(
-        status=200, body=json.dumps(processed_response), content_type="application/json"
-    )
\ No newline at end of file
+    return json_response([x for x in response], dumps=to_json)
