@@ -1,4 +1,3 @@
-import asyncio
 import base64
 from base64 import urlsafe_b64decode, urlsafe_b64encode
 from datetime import date, datetime
@@ -12,8 +11,8 @@ from iso8601 import parse_date
 from pydantic import BaseModel  # pylint: disable=no-name-in-module
 from typing_extensions import override
 
-from .objects import FaunaTime, Native, Query, Ref, SetRef
-from .query import Expr
+from .faunadb.objects import FaunaTime, Native, Query, Ref, SetRef
+from .faunadb.query import Expr
 
 T = TypeVar("T")
 
@@ -68,6 +67,7 @@ def to_json(dct, pretty=True, sort_keys=True):
         separators=(",", ":"),
         allow_nan=False,
         ensure_ascii=True,
+        exclude_none=True,
     )
 
 
@@ -87,7 +87,7 @@ class FaunaJSONEncoder(JSONEncoder):
             try:
                 _val = obj.decode()
             except:
-                _val = urlsafe_b64encode(obj).decode() # pylint: disable=all
+                _val = urlsafe_b64encode(obj).decode()  # pylint: disable=all
             return {"@bytes": _val}
         elif isinstance(obj, BaseModel):
             return obj.dict()
@@ -99,13 +99,12 @@ class FaunaJSONEncoder(JSONEncoder):
             return super().default(obj)
 
 
-
 class JSONModel(BaseModel):
     def to_dict(self, **kwargs):
         return parse_json(self.to_json(**kwargs))
 
     def to_json(self, **kwargs) -> str:
-        return to_json(super().dict(**kwargs))
+        return to_json(super().dict(exclude_none=True, **kwargs))
 
     @override
     def dict(self, **kwargs):
@@ -113,7 +112,7 @@ class JSONModel(BaseModel):
 
     @override
     def json(self, **kwargs) -> str:
-        return self.to_json(**kwargs)
+        return self.to_json(exclude_none=True, **kwargs)
 
 
 def jsonable_encoder(
@@ -229,5 +228,5 @@ def jsonable_encoder(
             skip_defaults=skip_defaults,
             custom_encoder=custom_encoder,
         )
-        
+
     return custom_encoder().default(obj)
