@@ -1,76 +1,51 @@
+from __future__ import annotations
+
 import asyncio
-import logging
-import os
-from collections import defaultdict
 from typing import Any, List, Optional, Type, TypeVar
 
+from dotenv import load_dotenv
+from pydantic import Field
 from pydantic.main import ModelMetaclass
 
-try:
-    import query as q
-
-except ImportError:
-    from . import query as q
-
-from dotenv import load_dotenv
-from pydantic import BaseModel  # pylint: disable=no-name-in-module
-
 from .client import FaunaClient
-from .errors import FaunaException
+from .faunadb import query as q
+from .faunadb.errors import FaunaException
 from .json import JSONModel
+from .utils import gen_emptystr, setup_logging
 
 load_dotenv()
 
 T = TypeVar("T", bound="FaunaModel")
 
-ModelType = Type[T]
-
-MaybeModel = Optional[ModelType]
-
-ModelTypeList = List[ModelType]
-
-ModelList = List[T]
 
 class FaunaModelMetaclass(ModelMetaclass):
-
-    def __new__(cls, name, bases, namespace, **kwargs):
-        new_cls = super().__new__(cls, name, bases, namespace, **kwargs)
+    def __new__(cls, name, bases, attrs):
+        new_cls = super().__new__(cls, name, bases, attrs)
         cls.Metadata.__subclasses__.append(new_cls)
         return new_cls
-    
+
     class Metadata:
         __subclasses__: List[Type[FaunaModel]] = []
-        
+
+
 class FaunaModel(JSONModel, metaclass=FaunaModelMetaclass):
-    
-    ref: Optional[str] = None
+    ref: str = Field(default_factory=gen_emptystr)
+    ts: str = Field(default_factory=gen_emptystr)
 
-    ts: Optional[str] = None
-    
+    @classmethod
+    @property
+    def logger(cls):
+        return setup_logging(cls.__name__)
 
-    def __init__(self, **data: Any):
-        super().__init__(**data)
-
-        for field in self.__fields__.values():
-            try:
-                one_of = field.field_info.extra.get("oneOf")
-
-                if isinstance(one_of, list):
-                    if data.get(field.name) not in one_of:
-                        raise ValueError(f"{field.name} must be one of {one_of}")
-
-            except Exception:
-                continue
-    
     @classmethod
     async def create_all(cls):
-        await asyncio.gather(*[model.provision() for model in cls.Metadata.__subclasses__])
-
+        await asyncio.gather(
+            *[model.provision() for model in cls.Metadata.__subclasses__]
+        )
 
     @classmethod
-    def client(cls) -> FaunaClient:
-        fauna_secret = os.getenv("FAUNA_SECRET")
-        return FaunaClient(secret=fauna_secret)
+    def client(cls):
+        return FaunaClient()
 
     @classmethod
     def q(cls):
@@ -84,7 +59,7 @@ class FaunaModel(JSONModel, metaclass=FaunaModelMetaclass):
             if not await _q(q.exists(q.collection(cls.__name__.lower()))):
                 await _q(q.create_collection({"name": cls.__name__.lower()}))
 
-                logging.info("Created collection %s", cls.__name__.lower())
+                cls.logger.info("Created collection %s", cls.__name__.lower())
 
             if not await _q(q.exists(q.index(cls.__name__.lower()))):
                 await _q(
@@ -96,8 +71,8 @@ class FaunaModel(JSONModel, metaclass=FaunaModelMetaclass):
                     )
                 )
 
-                logging.info("Created index %s", cls.__name__.lower())
-                
+                cls.logger.info("Created index %s", cls.__name__.lower())
+
             for field in cls.__fields__.values():
                 if field.field_info.extra.get("unique"):
                     if not await _q(
@@ -113,9 +88,11 @@ class FaunaModel(JSONModel, metaclass=FaunaModelMetaclass):
                                 }
                             )
                         )
-                        
-                        logging.info(
-                            "Created unique index %s_%s", cls.__name__.lower(), field.name
+
+                        cls.logger.info(
+                            "Created unique index %s_%s",
+                            cls.__name__.lower(),
+                            field.name,
                         )
                     continue
 
@@ -133,130 +110,92 @@ class FaunaModel(JSONModel, metaclass=FaunaModelMetaclass):
                             )
                         )
 
-                        logging.info("Created index %s_%s", cls.__name__.lower(), field.name)
+                        cls.logger.info(
+                            "Created index %s_%s", cls.__name__.lower(), field.name
+                        )
                         continue
 
             return True
 
-        except (FaunaException, KeyError, TypeError) as exc:
-            logging.error(exc)
+        except (FaunaException, KeyError, TypeError, ValueError, Exception) as exc:
+            cls.logger.error(exc.__class__.__name__)
+            cls.logger.error(exc)
 
             return False
 
     @classmethod
-    async def find_unique(cls:Type[T], field: str, value: Any) -> Optional[T]:
+    async def find_unique(cls: Type[T], **kwargs: Any) -> T:
         try:
+            field, value = list(kwargs.items())[0]
             data = await cls.q()(
                 q.get(q.match(q.index(f"{cls.__name__.lower()}_{field}_unique"), value))
             )
             return cls(
                 **{
-                    **data["data"], # type: ignore
-                    "ref": data["ref"]["@ref"]["id"], # type: ignore
-                    "ts": data["ts"] / 1000, # type: ignore
+                    **data["data"],  # type: ignore
+                    "ref": data["ref"]["@ref"]["id"],  # type: ignore
+                    "ts": data["ts"] / 1000,  # type: ignore
                 }
-            ) 
-
-        except (FaunaException, KeyError, TypeError) as exc:
-            logging.error(exc)
-            return None
-
-    @classmethod
-    async def find_many(cls:Type[T], field: str, value: Any) -> List[T]:
-        try:
-            _q = cls.q()
-
-            refs = (
-                await _q(
-                    q.paginate(
-                        q.match(q.index(f"{cls.__name__.lower()}_{field}"), value)
-                    )
-                )
-            )["data"] # type: ignore
-
-            collection_refs_map = defaultdict(list)
-
-            for ref in refs:
-                collection = q.collection(cls.__name__.lower())
-
-                collection_refs_map[collection].append( 
-                    q.ref(collection, ref["@ref"]["id"]) # type: ignore
-                )
-
-            data = await asyncio.gather(
-                *[
-                    _q(q.get(q.paginate(collection_refs)))
-                    for _, collection_refs in collection_refs_map.items()
-                ]
             )
 
-            flattened_data = [
-                item for collection_data in data for item in collection_data["data"]
-            ]
+        except (FaunaException, KeyError, TypeError, ValueError, Exception) as exc:
+            cls.logger.error(exc.__class__.__name__)
+            cls.logger.error(exc)
+            return None  # type: ignore
 
-            return [
-                cls(
-                    **{**d["data"], "ref": d["ref"]["@ref"]["id"], "ts": d["ts"] / 1000}
-                )
-                for d in flattened_data
-            ]
-
-        except (FaunaException, KeyError, TypeError) as exc:
-            logging.error(exc)
+    @classmethod
+    async def find_many(cls: Type[T], limit: int = 50, **kwargs: Any) -> List[T]:
+        try:
+            _q = cls.q()
+            field, value = list(kwargs.items())[0]
+            refs = await _q(
+                q.paginate(q.match(q.index(f"{cls.__name__.lower()}_{field}"), value))
+            )
+            refs = refs["data"][:limit]
+            return await asyncio.gather(*[cls.get(item["@ref"]["id"]) for item in refs])
 
+        except (FaunaException, KeyError, TypeError, ValueError, Exception) as exc:
+            cls.logger.error(exc.__class__.__name__)
+            cls.logger.error(exc)
             return []
 
     @classmethod
-    async def get(cls:Type[T], ref: str) -> Optional[T]:
+    async def get(cls: Type[T], ref: str) -> T:
         try:
             data = await cls.q()(q.get(q.ref(q.collection(cls.__name__.lower()), ref)))
             return cls(
                 **{
-                    **data["data"], # type: ignore
-                    "ref": data["ref"]["@ref"]["id"], # type: ignore
-                    "ts": data["ts"] / 1000, # type: ignore
+                    **data["data"],  # type: ignore
+                    "ref": data["ref"]["@ref"]["id"],  # type: ignore
+                    "ts": data["ts"] / 1000,  # type: ignore
                 }
             )
 
-        except (FaunaException, KeyError, TypeError) as exc:
-            logging.error(exc)
-            return None
+        except (FaunaException, KeyError, TypeError, ValueError, Exception) as exc:
+            cls.logger.error(exc.__class__.__name__)
+            cls.logger.error(exc)
+            raise exc
 
     @classmethod
-    async def all(cls:Type[T]) -> List[T]:
+    async def all(cls: Type[T], limit: int = 100, offset: int = 0) -> List[T]:
         try:
             _q = cls.q()
 
             query = q.paginate(q.match(q.index(f"{cls.__name__.lower()}")))
 
-            refs = (await _q(query))["data"] # type: ignore
+            refs = (await _q(query))["data"][offset:limit]
 
-            data = await asyncio.gather(
-                *[
-                    _q(
-                        q.get(
-                            q.ref(q.collection(cls.__name__.lower()), ref["@ref"]["id"])
-                        )
-                    )
-                    for ref in refs
-                ]
-            )
+            return await asyncio.gather(*[cls.get(item["@ref"]["id"]) for item in refs])
 
-            return [
-                cls(
-                    **{**d["data"], "ref": d["ref"]["@ref"]["id"], "ts": d["ts"] / 1000}
-                )
-                for d in data
-            ]
-
-        except (FaunaException, KeyError, TypeError) as exc:
-            logging.error(exc)
-
-            return []
+        except (FaunaException, KeyError, TypeError, ValueError, Exception) as exc:
+            cls.logger.error(exc.__class__.__name__)
+            cls.logger.error(exc)
+            raise exc
 
     @classmethod
-    async def delete_one(cls, field: str, value: Any) -> bool:
+    async def delete_one(cls: Type[T], **kwargs: Any) -> bool:
         try:
+            field, value = list(kwargs.items())[0]
             _q = cls.q()
 
             ref = await _q(
@@ -267,9 +206,9 @@ class FaunaModel(JSONModel, metaclass=FaunaModelMetaclass):
 
             return True
 
-        except (FaunaException, KeyError, TypeError) as exc:
-            logging.error(exc)
-
+        except (FaunaException, KeyError, TypeError, ValueError, Exception) as exc:
+            cls.logger.error(exc.__class__.__name__)
+            cls.logger.error(exc)
             return False
 
     @classmethod
@@ -279,17 +218,17 @@ class FaunaModel(JSONModel, metaclass=FaunaModelMetaclass):
 
             return True
 
-        except (FaunaException, KeyError, TypeError) as exc:
-            logging.error(exc)
-
+        except (FaunaException, KeyError, TypeError, ValueError, Exception) as exc:
+            cls.logger.error(exc.__class__.__name__)
+            cls.logger.error(exc)
             return False
 
-    async def create(self:T) -> Optional[T]:
+    async def create(self: T) -> T:
         try:
             for field in self.__fields__.values():
                 if field.field_info.extra.get("unique"):
                     instance = await self.find_unique(
-                        field.name, self.__dict__[field.name]
+                        **{field.name: self.__dict__[field.name]}
                     )
 
                     if instance is None:
@@ -304,45 +243,42 @@ class FaunaModel(JSONModel, metaclass=FaunaModelMetaclass):
                 )
             )
 
-            self.ref = data["ref"]["@ref"]["id"] # type: ignore
+            self.ref = data["ref"]["@ref"]["id"]  # type: ignore
 
-            self.ts = data["ts"] / 1000 # type: ignore
+            self.ts = data["ts"] / 1000  # type: ignore
             return self
 
-        except (FaunaException, KeyError, TypeError) as exc:
-            logging.error(exc)
-            return None
+        except (FaunaException, KeyError, TypeError, ValueError, Exception) as exc:
+            self.logger.error(exc.__class__.__name__)
+            self.logger.error(exc)
+            raise exc
 
     @classmethod
-    async def update(cls:Type[T], ref: str, **kwargs) -> T:
+    async def update(cls: Type[T], ref: str, **kwargs: Any) -> T:
         try:
-            instance = await cls.get(ref)
-
-            if isinstance(instance, cls):
-                instance_updated = await cls.q()(
-                    q.update(
-                        q.ref(q.collection(cls.__name__.lower()), ref),
-                        {"data": kwargs.get("kwargs", kwargs)},
-                    )
+            instance = await cls.q()(
+                q.update(
+                    q.ref(q.collection(cls.__name__.lower()), ref),
+                    {"data": kwargs.get("kwargs", kwargs)},
                 )
-                return cls(
-                    **{
-                        **instance_updated["data"], # type: ignore
-                        "ref": instance_updated["ref"]["@ref"]["id"], # type: ignore
-                        "ts": instance_updated["ts"] / 1000, # type: ignore
-                    }
-                )
-
-            else:
-                raise ValueError(f"Field {ref} not found")
-
+            )
+            return cls(
+                **{
+                    **instance["data"],  # type: ignore
+                    "ref": instance["ref"]["@ref"]["id"],  # type: ignore
+                    "ts": instance["ts"] / 1000,  # type: ignore
+                }
+            )
         except (FaunaException, KeyError, TypeError) as exc:
-            logging.error(exc)
+            cls.logger.error(exc.__class__.__name__)
+            cls.logger.error(exc)
+            raise ValueError(f"Field {ref} not found")  # pylint: disable=all
 
-            raise ValueError(f"Field {ref} not found") # pylint: disable=all
-
-    async def save(self:T) -> Optional[T]:
+    async def save(self: T) -> Optional[T]:
         if isinstance(self.ref, str) and len(self.ref) == 18:
             return await self.update(self.ref, kwargs=self.dict())
+        return await self.create()
 
-        return await self.create()
\ No newline at end of file
+    @classmethod
+    async def cleanup(cls):
+        await cls.client().cleanup()
